%ec4: PDIS (or BIS) code 
%Toy simulation for empirical comparison between variances of POE and quantile estimators.
%ed3: Parfor used.
function [norm_constant_IS_pdf] = BIS_num_integration_P()
global savefilename;

global p_var; %number of variablesclear
global target_probability;
global kappa; %non-linearity
global rho; %metamodeling quality
global delta; %difference in densities
global MtoN; %input sample size ratio;  M / N_T

global N_exp; % N_exp = Total_number_of_experiments_for_each_estimator
global N_T; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)


global mu;
global sigma; %In order to set it independent of x when performing experiment for delta


%savefilename = 'PDIS_code_ed4_rep200_P01.mat';

Time_Start = tic;


% Experiment setup
% p_var = 1; %number of variablesclear
% target_probability = 0.95;
% kappa = 2; %non-linearity
% rho = 0.6; %metamodeling quality
% delta = 1; %difference in densities
% MtoN = 0.3; %input sample size ratio;  M / N_T

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable


% Parameters
% N_exp = 200; % N_exp = Total_number_of_experiments_for_each_estimator
% N_T = 5000; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)
N_long = N_T;

%Parameter functions
%Note:  Need to input norm(x)  into the functions if |x| is wanted to be used for x.
%mu = (@(x) delta.*x  + (-0.5.*x).^kappa + (-0.5.*x).^(kappa+1) + (-0.5.*x).^(kappa+2) );
%sigma = (@(x) 1  + 0.5.*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x) );
%sigma_hat = (@(x) 1  + rho*(  0.5*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x)  ) );


%Compute a quantile based on a large-scale CMC
CMC_rep = 10^7;
randseed = 100;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);




% Calculate the normalizing constant of IS pdf using numerical integration
f = (@(x) mvnpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
s = (@(x) 1-normcdf(target_quantile, mu(x), sigma(x)));  

%kernel_meta = (@(x) f(x).* s_meta(x)); 

%kernel = (@(x) f(x).* s(x)); 
%norm_constant_IS_pdf = quadgk(kernel,-Inf,Inf); %value 1... why?


n = 1e16; %1e6; % more points = more accuracy. Relative error goes as 1/sqrt(n)
int_mcint = @(x) (f(x').* s(x'))'; %Transposes are necessary to be compatible with mcint().
opt.noerror = 0;
opt.tol = 1e-7; %2e-4;
opt.time = 60; %in seconds
opt.warnOff = 0;
opt.mcintPar = [int_mcint]; %[];
h.b = repmat([-Inf, Inf], [p_var 1]);
h.cond = {};
h.funcs = {@integrand_mcint}; %integrand_mcint.m  function file is created. %{(@(x) f(x').* sqrt(s_meta(x')./N_T + (1-1/N_T).*s_meta(x').^2))}; DOESN't work this way because an additional parameter needs to be passed.
mcint_output = mcint(h, n, opt);  %Note: the error is contained in mcint_output(1,1,2);  calculation code: getError(s,points,V) in mcint.m
norm_constant_IS_pdf = mcint_output(1,1,1);



old_digits = digits;   % For Variable-precision accuracy
digits(50);


disp('The results for BIS estimator:')


%   (2) Calculate the theoretical variance of the POE estimator: P(1-P)/N
theoretical_variance_of_POE_estimator = vpa(norm_constant_IS_pdf*(1-norm_constant_IS_pdf)/N_T); %based on numerical integration %vpa(target_probability*(1-target_probability)/N_T);
fprintf('Theoretical variance of POE estimator: %s \n', double(theoretical_variance_of_POE_estimator));









digits(old_digits); %Recover default accuracy: 32bit
Time_Elapsed = toc(Time_Start)      %#ok<NOPTS>
%save(savefilename)
end