%ED1: CMC code

global savefilename;

global p_var; %number of variablesclear
global target_probability;
global kappa; %non-linearity
global rho; %metamodeling quality
global delta; %difference in densities
global MtoN; %input sample size ratio;  M / N_T

global N_exp; % N_exp = Total_number_of_experiments_for_each_estimator
global N_T; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)

global mu;
global sigma; %In order to set it independent of x when performing experiment for delta



%savefilename = 'CMC_ed1_rep200_N5000.mat';

Time_Start = tic;
% Experiment setup
% p_var = 1; %number of variablesclear
% target_probability = 0.95;
% kappa = 2; %non-linearity
% rho = 0.6; %metamodeling quality
% delta = 1; %difference in densities
% MtoN = 0.3; %input sample size ratio;  M / N_T

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable

% Parameters
% N_exp = 200; % N_exp = Total_number_of_experiments_for_each_estimator
% N_T = 5000; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)


%Parameter functions
%Note:  Need to input norm(x)  into the functions if |x| is wanted to be used for x.
%mu = (@(x) delta.*x  + (-0.5.*x).^kappa + (-0.5.*x).^(kappa+1) + (-0.5.*x).^(kappa+2) );
%sigma = (@(x) 1  + 0.5.*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x) );
%sigma_hat = (@(x) 1  + rho*(  0.5*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x)  ) );


%Compute a quantile based on a large-scale CMC
CMC_rep = 10^7;
randseed = 100;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);



% Parallel running
n_parallel = 3; %Three (not Four) simulators are used.
% Reseed unpredictably the random number generator : http://blogs.mathworks.com/loren/2011/07/07/simpler-control-of-random-number-generation-in-matlab/
rng shuffle
spmd
s = RandStream.create('mrg32k3a','NumStreams',numlabs,'StreamIndices',labindex, 'Seed', 'shuffle');
RandStream.setGlobalStream(s);
end




old_digits = digits;   % For Variable-precision accuracy
digits(50);


%% ---------------------- Don't need this cell unless empirical standard error is needed


POE_estimate_of_exp = repmat(vpa(0.0),[1,N_exp]);
matlabpool(n_parallel)   

parfor index_experiment = 1:N_exp

    sample_x = mvnrnd(x_mu, x_sigma, N_T); 
    %norm_x = sqrt(sum(sample_x.^2,2));
    sample_y = normrnd(mu(sample_x), sigma(sample_x));   
    POE_estimate_of_exp(index_experiment) = mean(sample_y>target_quantile); 

fprintf('Experiment %d is conducted. \n', index_experiment);
end

matlabpool close

%%  ----------------------  Don't need the above cell unless the empirical standard error is needed.

disp('The results for CMC estimator:')

%   (2) Calculate the theoretical variance of the POE estimator: P(1-P)/N
theoretical_variance_of_POE_estimator = vpa(target_probability*(1-target_probability)/N_T);
fprintf('Theoretical variance of POE estimator: %s \n', double(theoretical_variance_of_POE_estimator));

%   (3) Calculate the emprical variance of the POE estimator based on the experiments
empirical_variance_of_POE_estimator = accurate_var(POE_estimate_of_exp) ;
fprintf('Sample variance of POE estimates : %s \n', double(empirical_variance_of_POE_estimator));


empirical_expectation_of_POE_estimator = accurate_mean(POE_estimate_of_exp);
standard_error_of_POE_estimator = accurate_std(POE_estimate_of_exp) ;
fprintf('Sample mean of POE estimates : %s \n', double(empirical_expectation_of_POE_estimator));
fprintf('Standard error of POE estimates : %s \n', double(standard_error_of_POE_estimator));




digits(old_digits); %Recover default accuracy: 32bit
Time_Elapsed = toc(Time_Start)      %#ok<NOPTS>
%save(savefilename)