function [x_final] = Kernel_based_IS_rnd(n_var, x_mu, x_sigma, kernel_meta, norm_constant_IS_pdf, N_long)     
    %'Acceptance rate =' norm_constant_IS_pdf
    %'Target sample size =' N_long
    %'Trial sample size =' N_long / norm_constant_IS_pdf
    
    %global p_var; %The body of a parfor-loop cannot contain global or persistent variable declarations. %http://stackoverflow.com/questions/12196122/globals-and-parfor
    
    N_tr = ceil(N_long/norm_constant_IS_pdf);  %Initial trial sample size    
    N_acc = 0;
    x_final = zeros(N_long, n_var);
    
    while(N_acc < N_long) 
        %Step 1: Generate X from q'(x)
        x = mvnrnd(x_mu, x_sigma, N_tr); %1D version:  normrnd(x_mu,x_sigma, [1,N_tr]);    

        %Step 2: Generate Y from U(0, q'(X))
        kernel_auxiliary_pdf = mvnpdf(x,x_mu,x_sigma);  %C_q' * q'(X) = f \sqrt(P(Y> l_T | X=x)= \lambda * exp(-(\lambda + l_T/2)*x)
        y = unifrnd(0,kernel_auxiliary_pdf);     

        %Step 3: If Y <= C_q * q(X), return X; otherwise, return to Step 1.
        kernel_IS_pdf = kernel_meta(x);  %kernel_IS_pdf = C_q * q(X) = f * \sqrt(        )

        %Accepted x
        x_acc = x(y<=kernel_IS_pdf,:);
        n_acc = size(x_acc,1);
        
        %Store the accepted x's
        if(n_acc > 0)
            if(N_acc+n_acc <= N_long)
                x_final(N_acc+1:N_acc+n_acc,:) = x_acc;            
                N_acc = N_acc + n_acc;
            else
                x_final(N_acc+1:N_long,:) = x_acc(1:N_long-N_acc,:);            
                N_acc = N_long;
            end
        end
        
        %Update N_tr
        N_tr = ceil((N_long-N_acc)/norm_constant_IS_pdf);  
    end
    
      
end


%% Below is the non-vectorized version

% function [random_variable] = Kernel_based_IS_rnd(x_mu, x_sigma, kernel)     
%     
% 
%     %Initialization for while-loop
%     y = 1;
%     kernel_IS_pdf =0;
%     
%     while(y > kernel_IS_pdf)
%         %Step 1: Generate X from q'(x)
%         x = mvnrnd(x_mu,x_sigma);          
% 
%         %Step 2: Generate Y from U(0, q'(X)) 
%         kernel_auxiliary_pdf = mvnpdf(x,x_mu,x_sigma);  %C_q' * q'(X) = f \sqrt(P(Y> l_T | X=x)= \lambda * exp(-(\lambda + l_T/2)*x)
%         y = unifrnd(0,kernel_auxiliary_pdf);     
% 
%         %Step 3: If Y <= C_q * q(X), return X; otherwise, return to Step 1.
%         kernel_IS_pdf = kernel(x);  %kernel_IS_pdf = C_q * q(X) = f * \sqrt(        )
%             %disp('repeat');
%     end
%     
%     random_variable = x;   
% end


% Test : generating r.v. from N = 1 case - 'the mean & std of resulting sample' should be the same with 'auxiliary_IS_dist_param'.
%                        from N = 1000000000  ' ' should be the same with 'Optimal_IS_dist_param = input_dist_param*target_probability'.
  