
global p_var; %number of variablesclear
global target_probability;
global kappa; %non-linearity
global rho; %metamodeling quality
global delta; %difference in densities
global MtoN; %input sample size ratio;  M / N_T

global N_exp; % N_exp = Total_number_of_experiments_for_each_estimator
global N_T; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)


global mu;
global sigma; %In order to set it independent of x when performing experiment for delta

global mu_hat;
global sigma_hat;

%% Plotting the base case

SetDefaultParameters()
%delta = -1;
%mu = (@(x) (0.95.*delta).*x.^2.*(1 + 0.5.*cos(10.*kappa.*x) + 0.5.*cos(20.*kappa.*x) ));

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable

CMC_rep = 10^5;

randseed = 100;
rng(randseed)
sample_x = mvnrnd(x_mu, x_sigma, CMC_rep);

sample_y = normrnd(mu(sample_x), 0); %sigma(sample_x));   
%sample_y = normrnd(mu(sample_x), sigma(sample_x));   
scatter(sample_x(:,1), sample_y,'.')
%scatter(sample_x, sample_y,'.')
xlim([-5,5]);
xlabel('X (input)'); 
ylabel('Y (output)');
title('Scatter plot for base parameters')

%title('Scatter plot for \beta = 1')




%% Plotting the optimal IS densities (3/30)
% Using x in [-5,5] and the density function q(x) = f(x) * s(x) / C_q
Methods = {'Original','SIS1', 'SIS2', 'BIS'};

%Compute a quantile based on a large-scale CMC
x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable
CMC_rep = 10^6;
randseed = 100;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);


f = (@(x) normpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
s = (@(x) 1-normcdf(target_quantile, mu(x), sigma(x)));   
%-------------------------- 04102014: To plot the IS densities based on the metamodel
rho = 1.00;
mu_hat = (@(x) (0.95.*delta).*x.^2.*(1 + 0.5.*rho.*cos(10.*kappa.*x) + 0.5.*rho.*cos(20.*kappa.*x) ));
sigma_hat = (@(x) 1 + 0.7.*abs(x) + 0.4.*rho.*cos(x) + 0.3.*rho.*cos(14.*x) );
s = (@(x) 1-normcdf(target_quantile, mu_hat(x), sigma_hat(x)));  
%--------------------------

xi = -5:0.001:5;

figure
xlim([-5,5]);
xlabel('x')
ylabel('Density')
for j = 1:length(Methods)
    Method = Methods{j};    
    switch Method
        case 'Original'
            kernel = f; 
            color = 'b';
            line_style = '-';
        case 'SIS1'
            kernel = (@(x) f(x).* sqrt(s(x)./N_T + (1-1/N_T).*s(x).^2));
            color = 'r';
            line_style = '--';
        case 'SIS2'
            kernel = (@(x) f(x).* sqrt(s(x))); 
            color = 'm';
            line_style = '-.';
        case 'BIS'
            kernel = (@(x) f(x).* s(x)); 
            color = 'g';
            line_style = ':';
    end
    
    norm_constant_IS_pdf = quadgk(kernel,-Inf,Inf); %value 1... why?
    pdf = kernel(xi)./norm_constant_IS_pdf;    

    hold on    
    plot(xi,pdf, 'Color',color, 'LineStyle',line_style)
    hold off
     
end

legend('Original','SIS1', 'SIS2', 'BIS')


%% Plotting the optimal IS densities (using the kernel density estimate --- obsolete )
% Note: Actually, I can directly plot x in [-5,5] and the density e.g., q(x) = f(x) * s(x) / C_q
Methods = {'Original','SIS1', 'SIS2', 'BIS'};

f = (@(x) normpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
s = (@(x) 1-normcdf(target_quantile, mu(x), sigma(x)));   

sample_x_size = 10^5;

figure
xlim([-5,5]);
xlabel('x')
ylabel('Estimated pdf')
for j = 1:length(Methods)
    Method = Methods{j};    
    switch Method
        case 'Original'
            kernel = f; 
            color = 'b';
            line_style = '-';
        case 'SIS1'
            kernel = (@(x) f(x).* sqrt(s(x)./N_T + (1-1/N_T).*s(x).^2));
            color = 'r';
            line_style = '--';
        case 'SIS2'
            kernel = (@(x) f(x).* sqrt(s(x))); 
            color = 'm';
            line_style = '-.';
        case 'BIS'
            kernel = (@(x) f(x).* s(x)); 
            color = 'g';
            line_style = ':';
    end
    
    norm_constant_IS_pdf = quadgk(kernel,-Inf,Inf); %value 1... why?
    sample_x = Kernel_based_IS_rnd(x_mu, x_sigma, kernel, norm_constant_IS_pdf, sample_x_size);

    hold on
    [pdf,xi] = ksdensity(sample_x);
    plot(xi,pdf, 'Color',color, 'LineStyle',line_style)
    hold off
     
end

legend('Original','SIS1', 'SIS2', 'BIS')





