%Edition 4: First version for numerical simulations for the paper revision
%Edition 3: SIS1 (OSIS) implementation for simulation study - For the paper
%Edition 2: To avoid the variability from using ratio estimator, the denominator of IS pdf is calculated using numerical integration.
%Edition 1: Op.alloc.IS pdf with "proposed pdf"

%Op.alloc.IS pdf with conventional pdf
%Edition 4: Allocation part is revised in order to ensure the unbiased estimation. 
%Edition 3: Semi-theoretic variance is omitted because of its inaccuracy. (maybe because of not enough sample size)
%Edition 2: Semi-theoretic variance is added.

global savefilename;

global p_var; %number of variablesclear
global target_probability;
global kappa; %non-linearity
global rho; %metamodeling quality
global delta; %difference in densities
global MtoN; %input sample size ratio;  M / N_T

global N_exp; % N_exp = Total_number_of_experiments_for_each_estimator
global N_T; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)


global mu;
global sigma; %In order to set it independent of x when performing experiment for delta

global mu_hat;
global sigma_hat;

global target_quantile; %for mcint

%savefilename = 'Optimal_allocation_IS_op_pdf_code_ed4_rep200_P01.mat';

Time_Start = tic;


% Experiment setup
% p_var = 1; %number of variablesclear
% target_probability = 0.85;
% kappa = 2; %non-linearity
% rho = 1; %metamodeling quality
% delta = 1; %difference in densities
% MtoN = 0.3; %input sample size ratio;  M / N_T

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable

% Parameters
% N_exp = 10; % N_exp = Total_number_of_experiments_for_each_estimator
% N_T = 500; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)
N_long = round(MtoN*N_T); %round is needed to make sure N_long is integer in MATLAB although divisible.  %round(N_T / N_short_ave); % the sample size from the optimal-allocation IS distribution (i.e., how many v_i are sampled in the o-a IS estimator)



%Parameter functions
%Note:  Need to input norm(x)  into the functions if |x| is wanted to be used for x.
%mu = (@(x) delta.*x  + (-0.5.*x).^kappa + (-0.5.*x).^(kappa+1) + (-0.5.*x).^(kappa+2) );
%sigma = (@(x) 1  + 0.5.*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x) );
%sigma_hat = (@(x) 1  + rho*(  0.5*cos(pi*kappa.*x)+ 0.25*cos(2*pi*kappa.*x) + 0.125*cos(3*pi*kappa.*x)  ) );
%mu_hat = (@(x) x.^2);

%Compute a quantile based on a large-scale CMC
CMC_rep = 10^7;
randseed = 100;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);




%Plot of integrand (i.e., C_q * q_sis1)
%x = (-3:0.01:3)'
%plot(x, integrad(x))

% Parallel running
n_parallel = 3; %Three (not Four) simulators are used.
% Reseed unpredictably the random number generator : http://blogs.mathworks.com/loren/2011/07/07/simpler-control-of-random-number-generation-in-matlab/
rng shuffle
spmd
s = RandStream.create('mrg32k3a','NumStreams',numlabs,'StreamIndices',labindex, 'Seed', 'shuffle');
RandStream.setGlobalStream(s);
end



% Calculate the normalizing constant of IS pdf using numerical integration
f = (@(x) mvnpdf(x,x_mu,x_sigma));   %f = (@(x) mvnpdf(x,x_mu(1:size(x,2)),x_sigma(1:size(x,2), 1:size(x,2)))); %Variation for 3d integration        
s = (@(x) 1-normcdf(target_quantile, mu(x), sigma(x)));
s_meta = (@(x) 1-normcdf(target_quantile, mu_hat(x), sigma_hat(x)));   

%norm2 = (@(x) sqrt(sum(x.^2,2)));
%s_meta = (@(x) 1-normcdf(target_quantile, mu(norm2(x)), sigma_hat(norm2(x))));     it leads to norm_constant_IS_pdf = 1
%kernel = (@(x) f(x).* sqrt(s(x)./N_T + (1-1/N_T).*s(x).^2));  %kernel is C_q * q_sis1
kernel_meta = (@(x) f(x).* sqrt(s_meta(x)./N_T + (1-1/N_T).*s_meta(x).^2));  %kernel is C_q * q_sis1
%norm_constant_IS_pdf = quadgk(kernel_meta,-Inf,Inf); %NOTE: 1D version


% 'integral3' does not work because it tries to eval the function with partial arguments?  %NOTE: Would work only for p_var = 3
% integrand = (@(x,y,z) f([x,y,z]).* sqrt(s_meta([x,y,z])./N_T + (1-1/N_T).*s_meta([x,y,z]).^2));  %Same function with kernel_meta except different arguments
% norm_constant_IS_pdf = integral3(integrand,-Inf,Inf,-Inf,Inf,-Inf,Inf); 


%mcint: http://www.mathworks.com/matlabcentral/fileexchange/12447-mcint
n = 1e16; %1e6; % more points = more accuracy. Relative error goes as 1/sqrt(n)
int_mcint = @(x) (f(x').* sqrt(s_meta(x')./N_T + (1-1/N_T).*s_meta(x').^2))'; %Transposes are necessary to be compatible with mcint().
%clearvars opt
%opt.maxArray = 1e10; %1e5;
opt.noerror = 0;
opt.tol = 1e-7; %2e-4;
opt.time = 60; %in seconds
opt.warnOff = 0;
opt.mcintPar = [int_mcint]; %[];
h.b = repmat([-Inf, Inf], [p_var 1]);
h.cond = {};
h.funcs = {@integrand_mcint}; %integrand_mcint.m  function file is created. %{(@(x) f(x').* sqrt(s_meta(x')./N_T + (1-1/N_T).*s_meta(x').^2))}; DOESN't work this way because an additional parameter needs to be passed.
mcint_output = mcint(h, n, opt);  %Note: the error is contained in mcint_output(1,1,2);  calculation code: getError(s,points,V) in mcint.m
norm_constant_IS_pdf = mcint_output(1,1,1);






old_digits = digits;   % For Variable-precision accuracy
digits(50);



POE_estimate_of_exp = repmat(vpa(0.0),[1,N_exp]);


matlabpool(n_parallel)   

parfor index_experiment = 1:N_exp
    %       (1-1) Sample x_i, i=1,...,N_long from IS distribution, q(x)

%     sample_x = zeros(1, N_long); %exprnd(Optimal_IS_dist_param, [1,N_long]);
%     for index_i = 1:N_long
%         sample_x(index_i) = Kernel_based_IS_rnd(x_mu, x_sigma, kernel_meta);
%     end

    %Vectorized 1D acceptance-rejection sampling
    n_var = p_var; 
    sample_x = Kernel_based_IS_rnd(n_var, x_mu, x_sigma, kernel_meta, norm_constant_IS_pdf, N_long);
    %ksdensity(sample_x)
    
    %       (1-2) Determine Optimal allocation of short-term sample size (Note: rounding error)
    allocation_proportion_for_obs_x = repmat(vpa(0.0),[1,N_long+1]);
    for index_i = 1:N_long
        P = s(sample_x(index_i,:));
        allocation_proportion_for_obs_x(index_i) = vpa(sqrt( (1-P)     / ( 1+(N_T-1)*P )    ) );  %proportional constant for the observed x_i =(f/q)*sqrt(P(Y>y_T | X=x)*(1-P(Y>y_T | X=x))) = sqrt( N*(exp(y_T*x) - 1) / (exp(y_T*x)+N-1)   )
    end
    alloc_normalizing_constant = vpa(0.0);
    for index_i = 1:N_long
        alloc_normalizing_constant = vpa(alloc_normalizing_constant + allocation_proportion_for_obs_x(index_i));    
    end
 


% Sort the proportions for x_i so that the most important (higest proportion) x_i has the first index.
[sorted_proportion, original_order]=sort(allocation_proportion_for_obs_x, 'descend');
temp_N_short_allocation = zeros(1, N_long); 
adjusted_normalizing_constant = alloc_normalizing_constant;
for index = (N_long:-1:2)
    remained_sample_size = N_T-sum(temp_N_short_allocation(index+1:N_long));   %N - [  sum_j=(k+1)^(N_long) N_j ]
    adjusted_normalizing_constant =  vpa(adjusted_normalizing_constant - sorted_proportion(index+1));
    unrounded_alloc_for_obs_x = vpa(remained_sample_size * sorted_proportion(index)/adjusted_normalizing_constant);
    if (double(unrounded_alloc_for_obs_x)<1)
        temp_N_short_allocation(index) = 1;
    else    %(double(unrounded_alloc_for_obs_x)>=1)
        temp_N_short_allocation(index) = round(double(unrounded_alloc_for_obs_x));
    end       
end 
temp_N_short_allocation(1) = N_T - sum(temp_N_short_allocation(2:N_long));    %N - [  sum_j=2^(N_long) N_j ]


N_short_allocation  = zeros(1, N_long);   % N_i : the allocated sample size for each stratum i
% Order to recover the original indice
for index = 1:N_long
    N_short_allocation(original_order(index)) = temp_N_short_allocation(index);
end


 

    %    (1-3) Sample N_i times from Y | X = x_i 
    %    (1-4) Sum up the I(Y_j^(i) > Y_T) over all j = 1..N_i and divide the sum by N_i to yield a short-term POE estimate
    Y_i_j_matrix = zeros([N_long,max(N_short_allocation)]); %Output j-th observation of Y for X = x_i
    Ind_Y_i_j_matrix = zeros([N_long,max(N_short_allocation)]); %Output Ind_Y_ij = I(Y_ij > y_T) for j-th observation of Y for X = x_i
    sum_for_short_term = zeros([1,N_long]);    %\sum_(j=1)^(N_i) I(Y_ij > y_T)
    term_for_short_term = repmat(vpa(0.0),[1,N_long]);   %f(x_i)/q(x_i) * (1/N_i) * \sum_(j=1)^(N_i) I(Y_ij > y_T)
    likelihood_at_x = repmat(vpa(0.0),[1,N_long]);  %=(f/q)*(1/C_q) = 1/sqrt( P/N + (1-1/N)*P^2 ) 
   % sum_of_likelihood = vpa(0.0);        %\sum_(i=1)^(N_long) (f/q)*(1/C_q) = sqrt( P/N + (1-1/N)*P^2 ) 
    for index_i = 1:N_long   % index i for each x_i
        for index_j = (1:N_short_allocation(index_i))  % index j for Y_j^(i) 
            Y_i_j_matrix(index_i,index_j)= CondYgivenXrnd( mu, sigma, sample_x(index_i,:) ); %NOTE: CondYgivenXrnd works only for 1-dimensional x
            Ind_Y_i_j_matrix(index_i,index_j)= ((Y_i_j_matrix(index_i,index_j)-target_quantile)>0);  %Y_T, the target quantile, is given,
            sum_for_short_term(index_i) = Ind_Y_i_j_matrix(index_i,index_j)+ sum_for_short_term(index_i);
        end
        likelihood_at_x(index_i) = vpa(1./sqrt(s_meta(sample_x(index_i,:))./N_T + (1-1/N_T).*s_meta(sample_x(index_i,:)).^2) ); %f(x_i)/q(x_i)*(1/C_q)
       % sum_of_likelihood = vpa(likelihood_at_x(index_i) + sum_of_likelihood);
        term_for_short_term(index_i) = vpa(likelihood_at_x(index_i) / N_short_allocation(index_i) * sum_for_short_term(index_i));          

    end
     
 
    
    %       (1-5) Calculate a POE estimate by summing up short-term POE estimate *(f(v_i)/q(v_i)) over all i = 1..N_long and divide the sum by N_long
    sum_for_long_term = vpa(0.0);
    for index_i = (1:N_long)  
       sum_for_long_term = vpa(sum_for_long_term + term_for_short_term(index_i));
    end
    
      POE_estimate_of_exp(index_experiment) = vpa(sum_for_long_term / N_long * norm_constant_IS_pdf);     %Note: (1/C_q)* N_long = expectation( sum_of_likelihood )

   

fprintf('Experiment %d is conducted. \n', index_experiment);


end

matlabpool close

disp('The results for SIS1 estimator:')


empirical_variance_of_POE_estimator_op_al_IS_op_pdf_IS_op_pdf = accurate_var(POE_estimate_of_exp) ;
fprintf('Sample variance of POE estimates : %s \n', double(empirical_variance_of_POE_estimator_op_al_IS_op_pdf_IS_op_pdf));


% Reference of necessary Integration:
%syms x a
%int(sqrt(exp(-a*x)*(1-exp(-a*x))),x)
%= -(asin(2/exp(a*x) - 1)/2 + (1/exp(a*x) - 1/exp(2*a*x))^(1/2))/a
%a = 3
%int(sqrt(exp(-a*x)*(1-exp(-a*x))),x, 0, Inf)
%ans = pi/6
 


%   (3) Calculate the emprical variance of the POE estimator based on the experiments

empirical_expectation_of_POE_estimator_op_alloc_IS_op_pdf = accurate_mean(POE_estimate_of_exp);
standard_error_of_POE_estimator = accurate_std(POE_estimate_of_exp) ;

fprintf('Sample mean of POE estimates : %s \n', double(empirical_expectation_of_POE_estimator_op_alloc_IS_op_pdf));
fprintf('Standard error of POE estimates : %s \n', double(standard_error_of_POE_estimator));





digits(old_digits); %Recover default accuracy: 32bit
Time_Elapsed = toc(Time_Start)      %#ok<NOPTS>

%save(savefilename)