%We want to plot the distribution of N_i to see the similarity between SIS1 and SIS2

global savefilename;

global p_var; %number of variablesclear
global target_probability;
global kappa; %non-linearity
global rho; %metamodeling quality
global delta; %difference in densities
global MtoN; %input sample size ratio;  M / N_T

global N_exp; % N_exp = Total_number_of_experiments_for_each_estimator
global N_T; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)


global mu;
global sigma; %In order to set it independent of x when performing experiment for delta

global mu_hat;
global sigma_hat;

%% Plotting s(x) vs. kernels of N*q_sis1 or q_sis2.
s = 0:0.001:1;
s1 = sqrt(s.*(1-s));
s2 = sqrt(s);
plot(s,s1, 'b')
hold on
plot(s,s2, 'r','LineStyle','--')
hold off
xlabel('s(x)')
ylabel('y')
hleg = legend('$y=\sqrt{s(x)[1-s(x)]}$','$y=\sqrt{s(x)}$', 'Location', 'Northwest');%'Best');
set(hleg, 'Interpreter', 'latex','fontsize',14)





%% Gathering N_i for the default setup with numerical example v6
SetDefaultParameters();
% delta = -1;
% mu = (@(x) (0.95.*delta).*x.^2.*(1 + 0.5.*cos(10.*kappa.*x) + 0.5.*cos(20.*kappa.*x) ));

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable

N_long = round(MtoN*N_T); %round is needed to make sure N_long is integer in MATLAB although divisible.  %round(N_T / N_short_ave); % the sample size from the optimal-allocation IS distribution (i.e., how many v_i are sampled in the o-a IS estimator)

CMC_rep = 10^6;
randseed = 100;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);


f = (@(x) normpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
s = (@(x) 1-normcdf(target_quantile, mu(x), sigma(x)));
s_meta = (@(x) 1-normcdf(target_quantile, mu_hat(x), sigma_hat(x)));   

kernel_meta = (@(x) f(x).* sqrt(s_meta(x)./N_T + (1-1/N_T).*s_meta(x).^2));  %kernel is C_q * q_sis1
norm_constant_IS_pdf = quadgk(kernel_meta,-Inf,Inf); %value 1... why?



old_digits = digits;   % For Variable-precision accuracy
digits(50);



  sample_x = Kernel_based_IS_rnd(x_mu, x_sigma, kernel_meta, norm_constant_IS_pdf, N_long);
    %ksdensity(sample_x)
    
    %       (1-2) Determine Optimal allocation of short-term sample size (Note: rounding error)
    allocation_proportion_for_obs_x = repmat(vpa(0.0),[1,N_long+1]);
    for index_i = 1:N_long
        P = s(sample_x(index_i));
        allocation_proportion_for_obs_x(index_i) = vpa(sqrt( (1-P)     / ( 1+(N_T-1)*P )    ) );  %proportional constant for the observed x_i =(f/q)*sqrt(P(Y>y_T | X=x)*(1-P(Y>y_T | X=x))) = sqrt( N*(exp(y_T*x) - 1) / (exp(y_T*x)+N-1)   )
    end
    alloc_normalizing_constant = vpa(0.0);
    for index_i = 1:N_long
        alloc_normalizing_constant = vpa(alloc_normalizing_constant + allocation_proportion_for_obs_x(index_i));    
    end
 


% Sort the proportions for x_i so that the most important (higest proportion) x_i has the first index.
[sorted_proportion, original_order]=sort(allocation_proportion_for_obs_x, 'descend');
temp_N_short_allocation = zeros(1, N_long); 
adjusted_normalizing_constant = alloc_normalizing_constant;
for index = (N_long:-1:2)
    remained_sample_size = N_T-sum(temp_N_short_allocation(index+1:N_long));   %N - [  sum_j=(k+1)^(N_long) N_j ]
    adjusted_normalizing_constant =  vpa(adjusted_normalizing_constant - sorted_proportion(index+1));
    unrounded_alloc_for_obs_x = vpa(remained_sample_size * sorted_proportion(index)/adjusted_normalizing_constant);
    if (double(unrounded_alloc_for_obs_x)<1)
        temp_N_short_allocation(index) = 1;
    else    %(double(unrounded_alloc_for_obs_x)>=1)
        temp_N_short_allocation(index) = round(double(unrounded_alloc_for_obs_x));
    end       
end 
temp_N_short_allocation(1) = N_T - sum(temp_N_short_allocation(2:N_long));    %N - [  sum_j=2^(N_long) N_j ]


N_short_allocation  = zeros(1, N_long);   % N_i : the allocated sample size for each stratum i
% Order to recover the original indice
for index = 1:N_long
    N_short_allocation(original_order(index)) = temp_N_short_allocation(index);
end



%% Plotting N_i 
scatter(sample_x, N_short_allocation)
xlim([-5 5])
xlabel('x')
ylabel('N_i')

ylim([0 8])